const q_builder = (db_name, username, pass = '', host = 'localhost', port = 3306) => {
    const mysql = require('mysql');
    let query = '';
    let selection = [];
    let whereClause = [];
    let joins = [];
    let orders = {};
    let groups = [];
    let tableFrom = "";
    let isCon = false;

    const _pool = mysql.createPool({
        host: host,
        port: port,
        database: db_name,
        user: username,
        password: pass
    });

    this.select = (select = '*') => {
        selection.push(select);
        return this;
    }

    this.from = (table) => {
        tableFrom = table;
        return this;
    }


    this.where = (column, value, operator = '=', string_value = true) => {
        if (whereClause.length == 0)
            whereClause.push({ col: column, value: value, operator: operator, tipe: ' ', string_value: string_value });
        else
            whereClause.push({ col: column, value: value, operator: operator, tipe: 'AND', string_value: string_value });
        return this;
    }

    this.or_where = (column, value, operator = '=', string_value = true) => {
        whereClause.push({ col: column, value: value, operator: operator, tipe: 'OR', string_value: string_value });
        return;
    }

    this.join = (table, on, tipe = 'INNER') => {
        joins.push({ table: table, on: on, tipe: tipe });
        return this;
    }

    this.order_by = (column, order = 'ASC') => {
        orders = { col: column, order: order };
        return this;
    }

    this.group_by = (column) => {
        groups.push(column);
        return this;
    }

    this.row = (callback, table = null, onError = null) => {
        if (table != null)
            tableFrom = table;

        if (onError == null || typeof (onError) != 'function')
            onError = () => { };

        let query = build_query();
        query += " LIMIT 1";
        _pool.getConnection((err, connection) => {
            if (err)
                console.log("Error Connection DB", err);

            connection.query(query, callback);
            connection.release();
        });
        _reset();

    }

    this.row_async = (table = null) => {
        if (table != null)
            tableFrom = table;

        let query = build_query();
        query += " LIMIT 1";
        _reset();
        return new Promise((resolve, reject)=>{
            _pool.query(query,  (error, elements)=>{
                if(error){
                    return reject(error);
                }
                return resolve(elements);
            });
            
        });   
    }
    this.results = (callback, table = null, onError = null) => {
        if (table != null)
            tableFrom = table;

        if (callback == null)
            callback = () => { };

        if (onError == null || typeof (onError) != 'function')
            onError = () => { };

        const query = build_query();
        _pool.getConnection((err, connection) => {
            if (err)
                console.log("Error Connection DB", err);

            connection.query(query, callback);
            connection.release();
        });
        _reset();

    }

    this.results_async = (table = null) => {
        if (table != null)
            tableFrom = table;

        const query = build_query();
        _reset();
        return new Promise((resolve, reject)=>{
            _pool.query(query,  (error, elements)=>{
                if(error){
                    return reject(error);
                }
                return resolve(elements);
            });
            
        });   
    }

    this.insert = (tabel, input, callback = null, onError = null) => {
        let query = 'INSERT INTO';
        if (Array.isArray(input)) {
            input.forEach((v, i) => {
                if (i == 0) {
                    query += ` ${tabel} (${Object.keys(v).join(', ')}) VALUES (${Object.keys(v).join(', ')})`
                } else {
                    query += `,(${Object.keys(v).join(', ')})`;
                }
            })
        } else {
            query = `INSERT INTO ${tabel} (${Object.keys(input).join(', ')}) VALUES (${Object.values(input).map(k => "'" + k + "'").join(', ')})`;
        }

        if (callback == null)
            callback = () => { };

        if (onError == null || typeof (onError) != 'function')
            onError = () => { };

        _pool.getConnection((err, connection) => {
            if (err)
                console.log("Error Connection DB", err);

            connection.query(query, callback);
            connection.release();
        });
        _reset();
    }

    this.insert_async = (tabel, input) => {
        let query = 'INSERT INTO';
        if (Array.isArray(input)) {
            input.forEach((v, i) => {
                if (i == 0) {
                    query += ` ${tabel} (${Object.keys(v).join(', ')}) VALUES (${Object.keys(v).join(', ')})`
                } else {
                    query += `,(${Object.keys(v).join(', ')})`;
                }
            })
        } else {
            query = `INSERT INTO ${tabel} (${Object.keys(input).join(', ')}) VALUES (${Object.values(input).map(k => "'" + k + "'").join(', ')})`;
        }
        _reset();
        return new Promise((resolve, reject)=>{
            _pool.query(query,  (error, elements)=>{
                if(error){
                    return reject(error);
                }
                return resolve(elements);
            });
            
        });   
    }

    this.update = (table, input, callback = null, onError = null) => {
        let query = `UPDATE ${table} SET`;
        Object.keys(input).forEach((k, i) => {
            if (i == 0)
                query += ` ${k} = '${input[k]}'`;
            else
                query += `, ${k} = '${input[k]}'`;
        });

        if (callback == null)
            callback = () => { };

        if (onError == null || typeof (onError) != 'function')
            onError = () => { };

        query = build_query(query);
        _pool.getConnection((err, connection) => {
            if (err)
                console.log("Error Connection DB", err);

            connection.query(query, callback);
            connection.release();
        });
        _reset();

    }
    this.update_async = (table, input) => {
        let query = `UPDATE ${table} SET`;
        Object.keys(input).forEach((k, i) => {
            if (i == 0)
                query += ` ${k} = '${input[k]}'`;
            else
                query += `, ${k} = '${input[k]}'`;
        });

        query = build_query(query);
        _reset();
        return new Promise((resolve, reject)=>{
            _pool.query(query,  (error, elements)=>{
                if(error){
                    return reject(error);
                }
                return resolve(elements);
            });
            
        });
    }

    this.delete = (table, callback = null, onError = null) => {
        let query = `DELETE FROM ${table}`;

        if (callback == null)
            callback = () => { };

        if (onError == null || typeof (onError) != 'function')
            onError = () => { };

        query = build_query(query);
        _pool.getConnection((err, connection) => {
            if (err)
                console.log("Error Connection DB", err);

            connection.query(query, callback);
            connection.release();
        });
        _reset();

    }
    this.delete_async = (table) => {
        let query = `DELETE FROM ${table}`;

       
        query = build_query(query);
        _reset();
        return new Promise((resolve, reject)=>{
            _pool.query(query,  (error, elements)=>{
                if(error){
                    return reject(error);
                }
                return resolve(elements);
            });
            
        });

    }
    const beforeExecute = () => {
        if (!isCon) {
            connection.connect(err => {
                if (err) {
                    throw console.log(err);
                }
                isCon = true;
            });
        }

    }

    const _reset = () => {
        query = '';
        selection = [];
        whereClause = [];
        joins = [];
        orders = {};
        groups = [];
        tableFrom = "";
    }

    const build_query = (query = 'select') => {
        let compiled_query = ``;

        if (query == 'select')
            compiled_query = `SELECT ${selection.join(', ')} FROM ${tableFrom}`;
        else
            compiled_query = query + ' ';

        if (joins.length > 0) {
            joins.forEach(j => {
                compiled_query += ` ${j.tipe} JOIN ${j.table} ON ${j.on}`;
            });
        }

        whereClause.forEach((w, i) => {
            if (i == 0){
                if(w.string_value)
                    compiled_query += ` WHERE ${w.col} ${w.operator} '${w.value}' `;
                else
                    compiled_query += ` WHERE ${w.col} ${w.operator} ${w.value} `;
            }
            else{
                if(w.string_value)
                    compiled_query += ` ${w.tipe} ${w.col} ${w.operator} '${w.value}' `;
                else
                    compiled_query += ` ${w.tipe} ${w.col} ${w.operator} ${w.value} `;
            }

        });

        if (groups.length != 0)
            compiled_query += ` GROUP By ${groups.join(", ")}`;

        if (Object.keys(orders).length != 0)
            compiled_query += ` ORDER BY ${orders.col} ${orders.order}`;

        return compiled_query;
    }

    return this;
}

module.exports = q_builder;